@@ -3,14 +3,14 @@
 # a release tarball.  Run from .release.sh, *before*
 # autogen.sh. 
 
-inmk=/usr/share/gettext/po/Makefile.in.in
+inmk=Makefile.in.in
 tmpmk=`mktemp /tmp/sitecopy.XXXXXX`
 pot=`mktemp /tmp/sitecopy.XXXXXX`
 trap 'rm -f $tmpmk $pot' TERM INT 0
 
 cd po
 
-lngs=`echo *.po | sed "s/.po//g"`
+lngs=`cat LINGUAS`
 for l in $lngs; do
     CATALOGS="$CATALOGS $l.gmo"
     POFILES="$POFILES $l.po"
@@ -38,6 +38,7 @@ s/@GMSGFMT@/msgfmt/g;
 s/@MSGFMT@/msgfmt/g;
 s/@XGETTEXT@/xgettext/g;
 s/@MSGMERGE@/msgmerge/g;
+s/@XGETTEXT_EXTRA_OPTIONS@/--msgid-bugs-address sitecopy@lists.manyfish.co.uk/g;
 s/: Makefile.*/:/g;
 s/\$(MAKE) update-gmo/echo Done/g;" $inmk > $tmpmk
 
