@@ -1,6 +1,6 @@
 /* 
    HTTP Authentication routines
-   Copyright (C) 1999-2006, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2008, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -45,8 +45,11 @@
 
 #ifdef HAVE_OPENSSL
 #include <openssl/rand.h>
+#elif defined(HAVE_GNUTLS)
+#include <gcrypt.h>
 #endif
 
+#include <errno.h>
 #include <time.h>
 
 #include "ne_md5.h"
@@ -74,10 +77,6 @@
 #include "ne_sspi.h"
 #endif
 
-#if defined(HAVE_GSSAPI) || defined(HAVE_SSPI)
-#include "ne_private.h" /* only need to extract proxy hostname */
-#endif
-
 #define HOOK_SERVER_ID "http://webdav.org/neon/hooks/server-auth"
 #define HOOK_PROXY_ID "http://webdav.org/neon/hooks/proxy-auth"
 
@@ -100,6 +99,8 @@ struct auth_handler {
 
     ne_auth_creds creds;
     void *userdata;
+    int attempt; /* number of invocations of this callback for
+                  * current request. */
     
     struct auth_handler *next;
 };
@@ -108,9 +109,7 @@ struct auth_handler {
 struct auth_challenge {
     const struct auth_protocol *protocol;
     struct auth_handler *handler;
-    const char *realm;
-    const char *nonce;
-    const char *opaque;
+    const char *realm, *nonce, *opaque, *domain;
     unsigned int stale; /* if stale=true */
     unsigned int got_qop; /* we were given a qop directive */
     unsigned int qop_auth; /* "auth" token in qop attrib */
@@ -120,15 +119,20 @@ struct auth_challenge {
 
 static const struct auth_class {
     const char *id, *req_hdr, *resp_hdr, *resp_info_hdr;
-    int status_code, fail_code;
+    int status_code; /* Response status-code to trap. */
+    int fail_code;   /* NE_* request to fail with. */
+    const char *error_noauth; /* Error message template use when
+                               * giving up authentication attempts. */
 } ah_server_class = {
     HOOK_SERVER_ID,
     "Authorization", "WWW-Authenticate", "Authentication-Info",
-    401, NE_AUTH
+    401, NE_AUTH,
+    N_("Could not authenticate to server: %s")
 }, ah_proxy_class = {
     HOOK_PROXY_ID,
     "Proxy-Authorization", "Proxy-Authenticate", "Proxy-Authentication-Info",
-    407, NE_PROXYAUTH 
+    407, NE_PROXYAUTH,
+    N_("Could not authenticate to proxy server: %s")
 };
 
 /* Authentication session state. */
@@ -142,8 +146,7 @@ typedef struct {
         AUTH_NOTCONNECT /* only in non-CONNECT responsees */
     } context;
     
-    /* Specifics for server/proxy auth. FIXME: need a better field
-     * name! */
+    /* Protocol type for server/proxy auth. */
     const struct auth_class *spec;
     
     /* The protocol used for this authentication session */
@@ -175,6 +178,8 @@ typedef struct {
     char *nonce;
     char *cnonce;
     char *opaque;
+    char **domains; /* list of paths given as domain. */
+    size_t ndomains; /* size of domains array */
     auth_qop qop;
     auth_algorithm alg;
     unsigned int nonce_count;
@@ -184,12 +189,6 @@ typedef struct {
     /* Temporary store for half of the Request-Digest
      * (an optimisation - used in the response-digest calculation) */
     struct ne_md5_ctx *stored_rdig;
-
-    /* Details of server... needed to reconstruct absoluteURI's when
-     * necessary */
-    const char *host;
-    const char *uri_scheme;
-    unsigned int port;
 } auth_session;
 
 struct auth_request {
@@ -210,6 +209,8 @@ struct auth_request {
 /* Used if this Authentication-Info may be sent for non-40[17]
  * response for this protocol. */
 #define AUTH_FLAG_VERIFY_NON40x (0x0002)
+/* Used for broken the connection-based auth schemes. */
+#define AUTH_FLAG_CONN_AUTH (0x0004)
 
 struct auth_protocol {
     unsigned id; /* public NE_AUTH_* id. */
@@ -220,24 +221,40 @@ struct auth_protocol {
     
     /* Parse the authentication challenge; returns zero on success, or
      * non-zero if this challenge be handled.  'attempt' is the number
-     * of times the request has been resent due to auth challenges. */
+     * of times the request has been resent due to auth challenges.
+     * On failure, challenge_error() should be used to append an error
+     * message to the error buffer 'errmsg'. */
     int (*challenge)(auth_session *sess, int attempt,
-                     struct auth_challenge *chall);
+                     struct auth_challenge *chall, ne_buffer **errmsg);
 
     /* Return the string to send in the -Authenticate request header:
      * (ne_malloc-allocated, NUL-terminated string) */
     char *(*response)(auth_session *sess, struct auth_request *req);
     
-    /* Parse a Authentication-Info response; returns NE_* error
-     * code. */
+    /* Parse a Authentication-Info response; returns NE_* error code
+     * on failure; on failure, the session error string must be
+     * set. */
     int (*verify)(struct auth_request *req, auth_session *sess,
                   const char *value);
     
     int flags; /* AUTH_FLAG_* flags */
 };
 
-/* Forward-declaration of protocols array. */
-static const struct auth_protocol protocols[];
+/* Helper function to append an error to the buffer during challenge
+ * handling.  Pass printf-style string.  *errmsg may be NULL and is
+ * allocated if necessary.  errmsg must be non-NULL. */
+static void challenge_error(ne_buffer **errmsg, const char *fmt, ...)
+    ne_attribute((format(printf, 2, 3)));
+
+/* Free the domains array, precondition sess->ndomains > 0. */
+static void free_domains(auth_session *sess)
+{
+    do {
+        ne_free(sess->domains[sess->ndomains - 1]);
+    } while (--sess->ndomains);
+    ne_free(sess->domains);
+    sess->domains = NULL;
+}
 
 static void clean_session(auth_session *sess) 
 {
@@ -252,6 +269,7 @@ static void clean_session(auth_session *sess)
         ne_md5_destroy_ctx(sess->stored_rdig);
         sess->stored_rdig = NULL;
     }
+    if (sess->ndomains) free_domains(sess);
 #ifdef HAVE_GSSAPI
     {
         unsigned int major;
@@ -280,65 +298,80 @@ static char *get_cnonce(void)
 
     hash = ne_md5_create_ctx();
 
-#ifdef HAVE_OPENSSL
-    if (RAND_status() == 1 && RAND_pseudo_bytes(data, sizeof data) >= 0)
+#ifdef HAVE_GNUTLS
+    if (1) {
+        gcry_create_nonce(data, sizeof data);
+        ne_md5_process_bytes(data, sizeof data, hash);
+    }
+    else
+#elif defined(HAVE_OPENSSL)
+    if (RAND_status() == 1 && RAND_pseudo_bytes(data, sizeof data) >= 0) {
 	ne_md5_process_bytes(data, sizeof data, hash);
-    else {
-#endif
-    /* Fallback sources of random data: all bad, but no good sources
-     * are available. */
-
-    /* Uninitialized stack data; yes, happy valgrinders, this is
-     * supposed to be here. */
-    ne_md5_process_bytes(data, sizeof data, hash);
-    
+    } 
+    else 
+#endif /* HAVE_OPENSSL */
+    {
+        /* Fallback sources of random data: all bad, but no good sources
+         * are available. */
+        
+        /* Uninitialized stack data; yes, happy valgrinders, this is
+         * supposed to be here. */
+        ne_md5_process_bytes(data, sizeof data, hash);
+        
+        {
 #ifdef HAVE_GETTIMEOFDAY
-    {
-	struct timeval tv;
-	if (gettimeofday(&tv, NULL) == 0)
-	    ne_md5_process_bytes(&tv, sizeof tv, hash);
-    }
+            struct timeval tv;
+            if (gettimeofday(&tv, NULL) == 0)
+                ne_md5_process_bytes(&tv, sizeof tv, hash);
 #else /* HAVE_GETTIMEOFDAY */
-    {
-	time_t t = time(NULL);
-	ne_md5_process_bytes(&t, sizeof t, hash);
-    }
+            time_t t = time(NULL);
+            ne_md5_process_bytes(&t, sizeof t, hash);
 #endif
-    {
+        }
+        {
 #ifdef WIN32
-	DWORD pid = GetCurrentThreadId();
+            DWORD pid = GetCurrentThreadId();
 #else
-	pid_t pid = getpid();
+            pid_t pid = getpid();
 #endif
-	ne_md5_process_bytes(&pid, sizeof pid, hash);
+            ne_md5_process_bytes(&pid, sizeof pid, hash);
+        }
     }
 
-#ifdef HAVE_OPENSSL
-    }
-#endif
-
     ne_md5_finish_ascii(hash, ret);
     ne_md5_destroy_ctx(hash);
 
     return ne_strdup(ret);
 }
 
-static int get_credentials(auth_session *sess, int attempt,
+/* Callback to retrieve user credentials for given session on given
+ * attempt (pre request) for given challenge.  Password is written to
+ * pwbuf (of size NE_ABUFSIZ.  On error, challenge_error() is used
+ * with errmsg. */
+static int get_credentials(auth_session *sess, ne_buffer **errmsg, int attempt,
                            struct auth_challenge *chall, char *pwbuf) 
 {
-    return chall->handler->creds(chall->handler->userdata, sess->realm, 
-                                 attempt, sess->username, pwbuf);
+    if (chall->handler->creds(chall->handler->userdata, sess->realm, 
+                              chall->handler->attempt++, sess->username, pwbuf) == 0) {
+        return 0;
+    } else {
+        challenge_error(errmsg, _("rejected %s challenge"), 
+                        chall->protocol->name);
+        return -1;
+    }
 }
 
 /* Examine a Basic auth challenge.
  * Returns 0 if an valid challenge, else non-zero. */
 static int basic_challenge(auth_session *sess, int attempt,
-                           struct auth_challenge *parms) 
+                           struct auth_challenge *parms,
+                           ne_buffer **errmsg) 
 {
     char *tmp, password[NE_ABUFSIZ];
 
     /* Verify challenge... must have a realm */
     if (parms->realm == NULL) {
+        challenge_error(errmsg, _("missing realm in Basic challenge"));
 	return -1;
     }
 
@@ -346,7 +379,7 @@ static int basic_challenge(auth_session *sess, int attempt,
     
     sess->realm = ne_strdup(parms->realm);
 
-    if (get_credentials(sess, attempt, parms, password)) {
+    if (get_credentials(sess, errmsg, attempt, parms, password)) {
 	/* Failed to get credentials */
 	return -1;
     }
@@ -419,8 +452,10 @@ static void make_gss_error(ne_buffer *buf, int *flag,
 }
 
 /* Continue a GSS-API Negotiate exchange, using input TOKEN if
- * non-NULL.  Returns non-zero on error. */
-static int continue_negotiate(auth_session *sess, const char *token)
+ * non-NULL.  Returns non-zero on error, in which case *errmsg is
+ * guaranteed to be non-NULL (i.e. an error message is set). */
+static int continue_negotiate(auth_session *sess, const char *token,
+                              ne_buffer **errmsg)
 {
     unsigned int major, minor;
     gss_buffer_desc input = GSS_C_EMPTY_BUFFER;
@@ -431,8 +466,7 @@ static int continue_negotiate(auth_session *sess, const char *token)
     if (token) {
         input.length = ne_unbase64(token, &bintoken);
         if (input.length == 0) {
-            NE_DEBUG(NE_DBG_HTTPAUTH, "gssapi: Invalid input [%s].\n",
-                     token);
+            challenge_error(errmsg, _("invalid Negotiate token"));
             return -1;
         }
         input.value = bintoken;
@@ -453,15 +487,12 @@ static int continue_negotiate(auth_session *sess, const char *token)
     if (bintoken) ne_free(bintoken);
 
     if (GSS_ERROR(major)) {
-        ne_buffer *err = ne_buffer_create();
         int flag = 0;
 
-        make_gss_error(err, &flag, major, GSS_C_GSS_CODE);
-        make_gss_error(err, &flag, minor, GSS_C_MECH_CODE);
-        NE_DEBUG(NE_DBG_HTTPAUTH, "gssapi: Error: %s\n", err->data);
-        ne_set_error(sess->sess, _("GSSAPI authentication error (%s)"), 
-                     err->data);
-        ne_buffer_destroy(err);
+        challenge_error(errmsg, _("GSSAPI authentication error: "));
+        make_gss_error(*errmsg, &flag, major, GSS_C_GSS_CODE);
+        make_gss_error(*errmsg, &flag, minor, GSS_C_MECH_CODE);
+
         return -1;
     }
 
@@ -471,7 +502,7 @@ static int continue_negotiate(auth_session *sess, const char *token)
         ret = 0;
     } 
     else {
-        NE_DEBUG(NE_DBG_HTTPAUTH, "gssapi: Init failure %d.\n", major);
+        challenge_error(errmsg, _("GSSAPI failure (code %u)"), major);
         ret = -1;
     }
 
@@ -495,18 +526,18 @@ static int continue_negotiate(auth_session *sess, const char *token)
 /* Process a Negotiate challange CHALL in session SESS; returns zero
  * if challenge is accepted. */
 static int negotiate_challenge(auth_session *sess, int attempt,
-                               struct auth_challenge *chall) 
+                               struct auth_challenge *chall,
+                               ne_buffer **errmsg) 
 {
     const char *token = chall->opaque;
 
     /* Respect an initial challenge - which must have no input token,
      * or a continuation - which must have an input token. */
     if (attempt == 0 || token) {
-        return continue_negotiate(sess, token);
+        return continue_negotiate(sess, token, errmsg);
     }
     else {
-        NE_DEBUG(NE_DBG_HTTPAUTH, "gssapi: Ignoring empty Negotiate "
-                 "challenge (attempt=%d).\n", attempt);
+        challenge_error(errmsg, _("ignoring empty Negotiate continuation"));
         return -1;
     }
 }
@@ -518,9 +549,11 @@ static int verify_negotiate_response(struct auth_request *req, auth_session *ses
     char *duphdr = ne_strdup(hdr);
     char *sep, *ptr = strchr(duphdr, ' ');
     int ret;
+    ne_buffer *errmsg = NULL;
 
-    if (strncmp(hdr, "Negotiate", ptr - hdr) != 0) {
-        NE_DEBUG(NE_DBG_HTTPAUTH, "gssapi: Not a Negotiate response!\n");
+    if (strncmp(hdr, "Negotiate", ptr - duphdr) != 0) {
+        ne_set_error(sess->sess, _("Negotiate response verification failed: "
+                                   "invalid response header token"));
         ne_free(duphdr);
         return NE_ERROR;
     }
@@ -539,23 +572,28 @@ static int verify_negotiate_response(struct auth_request *req, auth_session *ses
         *sep = '\0';
 
     NE_DEBUG(NE_DBG_HTTPAUTH, "gssapi: Negotiate response token [%s]\n", ptr);
-    ret = continue_negotiate(sess, ptr);
+    ret = continue_negotiate(sess, ptr, &errmsg);
     if (ret) {
-        NE_DEBUG(NE_DBG_HTTPAUTH, "gssapi: Mutual auth failed.\n");
+        ne_set_error(sess->sess, _("Negotiate response verification failure: %s"),
+                     errmsg->data);
     }
+
+    if (errmsg) ne_buffer_destroy(errmsg);
     ne_free(duphdr);
+
     return ret ? NE_ERROR : NE_OK;
 }
 #endif
 
 #ifdef HAVE_SSPI
-static char *request_sspi(auth_session *sess) 
+static char *request_sspi(auth_session *sess, struct auth_request *request) 
 {
     return ne_concat(sess->protocol->name, " ", sess->sspi_token, "\r\n", NULL);
 }
 
 static int sspi_challenge(auth_session *sess, int attempt,
-                          struct auth_challenge *parms) 
+                          struct auth_challenge *parms,
+                          ne_buffer **errmsg) 
 {
     int ntlm = ne_strcasecmp(parms->protocol->name, "NTLM") == 0;
     int status;
@@ -564,8 +602,14 @@ static int sspi_challenge(auth_session *sess, int attempt,
     NE_DEBUG(NE_DBG_HTTPAUTH, "auth: SSPI challenge.\n");
     
     if (!sess->sspi_context) {
-        status = ne_sspi_create_context(&sess->sspi_context,
-                                        sess->sess->server.hostname, ntlm);
+        ne_uri uri = {0};
+
+        ne_fill_server_uri(sess->sess, &uri);
+
+        status = ne_sspi_create_context(&sess->sspi_context, uri.host, ntlm);
+
+        ne_uri_free(&uri);
+
         if (status) {
             return status;
         }
@@ -584,48 +628,125 @@ static int sspi_challenge(auth_session *sess, int attempt,
 }
 #endif
 
+/* Parse the "domain" challenge parameter and set the domains array up
+ * in the session appropriately. */
+static int parse_domain(auth_session *sess, const char *domain)
+{
+    char *cp = ne_strdup(domain), *p = cp;
+    ne_uri base;
+    int invalid = 0;
+
+    memset(&base, 0, sizeof base);
+    ne_fill_server_uri(sess->sess, &base);
+
+    do {
+        char *token = ne_token(&p, ' ');
+        ne_uri rel, absolute;
+        
+        if (ne_uri_parse(token, &rel) == 0) {
+            /* Resolve relative to the Request-URI. */
+            ne_uri_resolve(&base, &rel, &absolute);
+
+            base.path = absolute.path;
+            
+            /* Ignore URIs not on this server. */
+            if (absolute.path && ne_uri_cmp(&absolute, &base) == 0) {
+                sess->domains = ne_realloc(sess->domains, 
+                                           ++sess->ndomains *
+                                           sizeof(*sess->domains));
+                sess->domains[sess->ndomains - 1] = absolute.path;
+                NE_DEBUG(NE_DBG_HTTPAUTH, "auth: Using domain %s from %s\n",
+                         absolute.path, token);
+                absolute.path = NULL;
+            }
+            else {
+                NE_DEBUG(NE_DBG_HTTPAUTH, "auth: Ignoring domain %s\n",
+                         token);
+            }
+
+            ne_uri_free(&absolute);
+        }
+        else {
+            invalid = 1;
+        }
+        
+        ne_uri_free(&rel);
+        
+    } while (p && !invalid);
+
+    if (invalid && sess->ndomains) {
+        free_domains(sess);
+    }
+
+    ne_free(cp);
+    base.path = NULL;
+    ne_uri_free(&base);
+
+    return invalid;
+}
+
 /* Examine a digest challenge: return 0 if it is a valid Digest challenge,
  * else non-zero. */
 static int digest_challenge(auth_session *sess, int attempt,
-                            struct auth_challenge *parms) 
+                            struct auth_challenge *parms,
+                            ne_buffer **errmsg) 
 {
     char password[NE_ABUFSIZ];
 
     if (parms->alg == auth_alg_unknown) {
-        ne_set_error(sess->sess, _("Unknown algorithm in Digest "
-                                   "authentication challenge"));
+        challenge_error(errmsg, _("unknown algorithm in Digest challenge"));
         return -1;
     }
     else if (parms->alg == auth_alg_md5_sess && !parms->qop_auth) {
-        ne_set_error(sess->sess, _("Incompatible algorithm in Digest "
-                                   "authentication challenge"));
+        challenge_error(errmsg, _("incompatible algorithm in Digest challenge"));
         return -1;
     }
     else if (parms->realm == NULL || parms->nonce == NULL) {
-	NE_DEBUG(NE_DBG_HTTPAUTH, "auth: Digest challenge missing parms.\n");
-        ne_set_error(sess->sess, _("Missing nonce or realm in Digest "
-                                   "authentication challenge"));
+        challenge_error(errmsg, _("missing parameter in Digest challenge"));
 	return -1;
     }
+    else if (parms->stale && sess->nonce_count == 0) {
+        challenge_error(errmsg, _("initial Digest challenge was stale"));
+        return -1;
+    }
+    else if (parms->stale && (sess->alg != parms->alg
+                              || strcmp(sess->realm, parms->realm))) {
+        /* With stale=true the realm and algorithm cannot change since these
+         * require re-hashing H(A1) which defeats the point. */
+        challenge_error(errmsg, _("stale Digest challenge with new algorithm or realm"));
+        return -1;
+    }
 
     if (!parms->stale) {
-	/* Forget the old session details */
-	clean_session(sess);
+        /* Non-stale challenge: clear session and request credentials. */
+        clean_session(sess);
 
-	sess->realm = ne_strdup(parms->realm);
+        /* The domain paramater must be parsed after the session is
+         * cleaned; ignore domain for proxy auth. */
+        if (parms->domain && sess->spec == &ah_server_class
+            && parse_domain(sess, parms->domain)) {
+            challenge_error(errmsg, _("could not parse domain in Digest challenge"));
+            return -1;
+        }
 
-	/* Not a stale response: really need user authentication */
-	if (get_credentials(sess, attempt, parms, password)) {
-	    /* Failed to get credentials */
-	    return -1;
-	}
+        sess->realm = ne_strdup(parms->realm);
+        sess->alg = parms->alg;
+        sess->cnonce = get_cnonce();
+
+        if (get_credentials(sess, errmsg, attempt, parms, password)) {
+            /* Failed to get credentials */
+            return -1;
+        }
     }
-    sess->alg = parms->alg;
+    else {
+        /* Stale challenge: accept a new nonce or opaque. */
+        if (sess->nonce) ne_free(sess->nonce);
+        if (sess->opaque && parms->opaque) ne_free(sess->opaque);
+    }
+    
     sess->nonce = ne_strdup(parms->nonce);
-    sess->cnonce = get_cnonce();
-    /* TODO: add domain handling. */
-    if (parms->opaque != NULL) {
-	sess->opaque = ne_strdup(parms->opaque); /* don't strip the quotes */
+    if (parms->opaque) {
+	sess->opaque = ne_strdup(parms->opaque);
     }
     
     if (parms->got_qop) {
@@ -681,6 +802,34 @@ static int digest_challenge(auth_session *sess, int attempt,
     return 0;
 }
 
+/* Returns non-zero if given Request-URI is inside the authentication
+ * domain defined for the session. */
+static int inside_domain(auth_session *sess, const char *req_uri)
+{
+    int inside = 0;
+    size_t n;
+    ne_uri uri;
+    
+    /* Parse the Request-URI; it will be an absoluteURI if using a
+     * proxy, and possibly '*'. */
+    if (strcmp(req_uri, "*") == 0 || ne_uri_parse(req_uri, &uri) != 0) {
+        /* Presume outside the authentication domain. */
+        return 0;
+    }
+
+    for (n = 0; n < sess->ndomains && !inside; n++) {
+        const char *d = sess->domains[n];
+        
+        inside = strncmp(uri.path, d, strlen(d)) == 0;
+    }
+    
+    NE_DEBUG(NE_DBG_HTTPAUTH, "auth: '%s' is inside auth domain: %d.\n", 
+             uri.path, inside);
+    ne_uri_free(&uri);
+    
+    return inside;
+}            
+
 /* Return Digest authentication credentials header value for the given
  * session. */
 static char *request_digest(auth_session *sess, struct auth_request *req) 
@@ -691,6 +840,12 @@ static char *request_digest(auth_session *sess, struct auth_request *req)
     const char *qop_value = "auth"; /* qop-value */
     ne_buffer *ret;
 
+    /* Do not submit credentials if an auth domain is defined and this
+     * request-uri fails outside it. */
+    if (sess->ndomains && !inside_domain(sess, req->uri)) {
+        return NULL;
+    }
+
     /* Increase the nonce-count */
     if (sess->qop != auth_qop_none) {
 	sess->nonce_count++;
@@ -868,12 +1023,7 @@ static int verify_digest_response(struct auth_request *req, auth_session *sess,
 	    cnonce = val;
 	} else if (ne_strcasecmp(key, "nc") == 0) { 
 	    nc = val;
-	    if (sscanf(val, "%x", &nonce_count) != 1) {
-		NE_DEBUG(NE_DBG_HTTPAUTH, "Couldn't find nonce count.\n");
-	    } else {
-		NE_DEBUG(NE_DBG_HTTPAUTH, "Got nonce_count: %u\n", nonce_count);
-	    }
-	}
+        }
     }
 
     if (qop == auth_qop_none) {
@@ -891,14 +1041,27 @@ static int verify_digest_response(struct auth_request *req, auth_session *sess,
         ne_set_error(sess->sess, _("Digest mutual authentication failure: "
                                    "client nonce mismatch"));
     }
-    else if (nonce_count != sess->nonce_count) {
-        ret = NE_ERROR;
-        ne_set_error(sess->sess, _("Digest mutual authentication failure: "
-                                   "nonce count mismatch (%u not %u)"),
-                     nonce_count, sess->nonce_count);
+    else if (nc) {
+        char *ptr;
+        
+        errno = 0;
+        nonce_count = strtoul(nc, &ptr, 16);
+        if (*ptr != '\0' || errno) {
+            ret = NE_ERROR;
+            ne_set_error(sess->sess, _("Digest mutual authentication failure: "
+                                       "could not parse nonce count"));
+        }
+        else if (nonce_count != sess->nonce_count) {
+            ret = NE_ERROR;
+            ne_set_error(sess->sess, _("Digest mutual authentication failure: "
+                                       "nonce count mismatch (%u not %u)"),
+                         nonce_count, sess->nonce_count);
+        }
     }
-    else {
-        /* Verify the response-digest field */
+
+    /* Finally, for qop=auth cases, if everything else is OK, verify
+     * the response-digest field. */    
+    if (qop == auth_qop_auth && ret == NE_OK) {
         struct ne_md5_ctx *a2;
         char a2_md5_ascii[33], rdig_md5_ascii[33];
 
@@ -941,9 +1104,9 @@ static int verify_digest_response(struct auth_request *req, auth_session *sess,
     /* Check for a nextnonce */
     if (nextnonce != NULL) {
 	NE_DEBUG(NE_DBG_HTTPAUTH, "auth: Found nextnonce of [%s].\n", nextnonce);
-	if (sess->nonce != NULL)
-	    ne_free(sess->nonce);
+        ne_free(sess->nonce);
 	sess->nonce = ne_strdup(nextnonce);
+        sess->nonce_count = 0;
     }
 
     ne_free(hdr);
@@ -951,6 +1114,29 @@ static int verify_digest_response(struct auth_request *req, auth_session *sess,
     return ret;
 }
 
+static const struct auth_protocol protocols[] = {
+    { NE_AUTH_BASIC, 10, "Basic",
+      basic_challenge, request_basic, NULL,
+      0 },
+    { NE_AUTH_DIGEST, 20, "Digest",
+      digest_challenge, request_digest, verify_digest_response,
+      0 },
+#ifdef HAVE_GSSAPI
+    { NE_AUTH_NEGOTIATE, 30, "Negotiate",
+      negotiate_challenge, request_negotiate, verify_negotiate_response,
+      AUTH_FLAG_OPAQUE_PARAM|AUTH_FLAG_VERIFY_NON40x|AUTH_FLAG_CONN_AUTH },
+#endif
+#ifdef HAVE_SSPI
+    { NE_AUTH_NEGOTIATE, 30, "NTLM",
+      sspi_challenge, request_sspi, NULL,
+      AUTH_FLAG_OPAQUE_PARAM|AUTH_FLAG_VERIFY_NON40x|AUTH_FLAG_CONN_AUTH },
+    { NE_AUTH_NEGOTIATE, 30, "Negotiate",
+      sspi_challenge, request_sspi, NULL,
+      AUTH_FLAG_OPAQUE_PARAM|AUTH_FLAG_VERIFY_NON40x|AUTH_FLAG_CONN_AUTH },
+#endif
+    { 0 }
+};
+
 /* Insert a new auth challenge for protocol 'proto' in list of
  * challenges 'list'.  The challenge list is kept in sorted order of
  * strength, with highest strength first. */
@@ -980,6 +1166,25 @@ static struct auth_challenge *insert_challenge(struct auth_challenge **list,
     return ret;
 }
 
+static void challenge_error(ne_buffer **errbuf, const char *fmt, ...)
+{
+    char err[128];
+    va_list ap;
+    size_t len;
+    
+    va_start(ap, fmt);
+    len = ne_vsnprintf(err, sizeof err, fmt, ap);
+    va_end(ap);
+    
+    if (*errbuf == NULL) {
+        *errbuf = ne_buffer_create();
+        ne_buffer_append(*errbuf, err, len);
+    }
+    else {
+        ne_buffer_concat(*errbuf, ", ", err, NULL);
+    }
+}
+
 /* Passed the value of a "(Proxy,WWW)-Authenticate: " header field.
  * Returns 0 if valid challenge was accepted; non-zero if no valid
  * challenge was found. */
@@ -988,6 +1193,7 @@ static int auth_challenge(auth_session *sess, int attempt,
 {
     char *pnt, *key, *val, *hdr, sep;
     struct auth_challenge *chall = NULL, *challenges = NULL;
+    ne_buffer *errmsg = NULL;
 
     pnt = hdr = ne_strdup(value); 
 
@@ -1014,8 +1220,9 @@ static int auth_challenge(auth_session *sess, int attempt,
             }
 
             if (proto == NULL) {
-		NE_DEBUG(NE_DBG_HTTPAUTH, "auth: Ignoring '%s' challenge.\n", key);
+                /* Ignore this challenge. */
                 chall = NULL;
+                challenge_error(&errmsg, _("ignored %s challenge"), key);
                 continue;
 	    }
             
@@ -1071,6 +1278,9 @@ static int auth_challenge(auth_session *sess, int attempt,
             
             chall->got_qop = chall->qop_auth;
 	}
+        else if (ne_strcasecmp(key, "domain") == 0) {
+            chall->domain = val;
+        }
     }
     
     sess->protocol = NULL;
@@ -1080,7 +1290,9 @@ static int auth_challenge(auth_session *sess, int attempt,
     for (chall = challenges; chall != NULL; chall = chall->next) {
         NE_DEBUG(NE_DBG_HTTPAUTH, "auth: Trying %s challenge...\n",
                  chall->protocol->name);
-        if (chall->protocol->challenge(sess, attempt, chall) == 0) {
+        if (chall->protocol->challenge(sess, attempt, chall, &errmsg) == 0) {
+            NE_DEBUG(NE_DBG_HTTPAUTH, "auth: Accepted %s challenge.\n", 
+                     chall->protocol->name);
             sess->protocol = chall->protocol;
             break;
         }
@@ -1088,6 +1300,8 @@ static int auth_challenge(auth_session *sess, int attempt,
 
     if (!sess->protocol) {
         NE_DEBUG(NE_DBG_HTTPAUTH, "auth: No challenges accepted.\n");
+        ne_set_error(sess->sess, _(sess->spec->error_noauth),
+                     errmsg ? errmsg->data : _("could not parse challenge"));
     }
 
     while (challenges != NULL) {
@@ -1097,6 +1311,7 @@ static int auth_challenge(auth_session *sess, int attempt,
     }
 
     ne_free(hdr);
+    if (errmsg) ne_buffer_destroy(errmsg);
 
     return !(sess->protocol != NULL);
 }
@@ -1111,6 +1326,7 @@ static void ah_create(ne_request *req, void *session, const char *method,
         (is_connect && sess->context == AUTH_CONNECT) ||
         (!is_connect && sess->context == AUTH_NOTCONNECT)) {
         struct auth_request *areq = ne_calloc(sizeof *areq);
+        struct auth_handler *hdl;
         
         NE_DEBUG(NE_DBG_HTTPAUTH, "ah_create, for %s\n", sess->spec->resp_hdr);
         
@@ -1119,6 +1335,12 @@ static void ah_create(ne_request *req, void *session, const char *method,
         areq->request = req;
         
         ne_set_request_private(req, sess->spec->id, areq);
+
+        /* For each new request, reset the attempt counter in every
+         * registered handler. */
+        for (hdl = sess->handlers; hdl; hdl = hdl->next) {
+            hdl->attempt = 0;
+        }
     }
 }
 
@@ -1140,7 +1362,6 @@ static void ah_pre_send(ne_request *r, void *cookie, ne_buffer *request)
 	    ne_buffer_concat(request, sess->spec->req_hdr, ": ", value, NULL);
 	    ne_free(value);
 	}
-
     }
 
 }
@@ -1180,8 +1401,8 @@ static int ah_post_send(ne_request *req, void *cookie, const ne_status *status)
         && (sess->protocol->flags & AUTH_FLAG_VERIFY_NON40x) == 0) {
         ret = sess->protocol->verify(areq, sess, auth_info_hdr);
     }
-    else if (sess->protocol
-             && sess->protocol->flags && AUTH_FLAG_VERIFY_NON40x
+    else if (sess->protocol && sess->protocol->verify
+             && (sess->protocol->flags & AUTH_FLAG_VERIFY_NON40x) 
              && (status->klass == 2 || status->klass == 3)
              && auth_hdr) {
         ret = sess->protocol->verify(areq, sess, auth_hdr);
@@ -1198,6 +1419,12 @@ static int ah_post_send(ne_request *req, void *cookie, const ne_status *status)
 	    clean_session(sess);
 	    ret = sess->spec->fail_code;
 	}
+        
+        /* Set or clear the conn-auth flag according to whether this
+         * was an accepted challenge for a borked protocol. */
+        ne_set_session_flag(sess->sess, NE_SESSFLAG_CONNAUTH,
+                            sess->protocol 
+                            && (sess->protocol->flags & AUTH_FLAG_CONN_AUTH));
     }
 #ifdef HAVE_SSPI
     else if (sess->sspi_context) {
@@ -1246,6 +1473,18 @@ static void auth_register(ne_session *sess, int isproxy, unsigned protomask,
     auth_session *ahs;
     struct auth_handler **hdl;
 
+    /* Handle the _ALL and _DEFAULT protocol masks: */
+    if (protomask == NE_AUTH_ALL) {
+        protomask |= NE_AUTH_BASIC | NE_AUTH_DIGEST | NE_AUTH_NEGOTIATE;
+    }
+    else if (protomask == NE_AUTH_DEFAULT) {
+        protomask |= NE_AUTH_BASIC | NE_AUTH_DIGEST;
+        
+        if (strcmp(ne_get_scheme(sess), "https") == 0 || isproxy) {
+            protomask |= NE_AUTH_NEGOTIATE;
+        }
+    }
+
     ahs = ne_get_session_private(sess, id);
     if (ahs == NULL) {
         ahs = ne_calloc(sizeof *ahs);
@@ -1271,14 +1510,22 @@ static void auth_register(ne_session *sess, int isproxy, unsigned protomask,
 
 #ifdef HAVE_GSSAPI
     if (protomask & NE_AUTH_NEGOTIATE && ahs->gssname == GSS_C_NO_NAME) {
-        get_gss_name(&ahs->gssname, (isproxy ? sess->proxy.hostname 
-                                     : sess->server.hostname));
+        ne_uri uri = {0};
+        
+        if (isproxy)
+            ne_fill_proxy_uri(sess, &uri);
+        else
+            ne_fill_server_uri(sess, &uri);
+
+        get_gss_name(&ahs->gssname, uri.host);
+
+        ne_uri_free(&uri);
     }
 #endif
 
     /* Find the end of the handler list, and add a new one. */
     hdl = &ahs->handlers;
-    while (*hdl && (*hdl)->next)
+    while (*hdl)
         hdl = &(*hdl)->next;
         
     *hdl = ne_malloc(sizeof **hdl);
@@ -1286,54 +1533,19 @@ static void auth_register(ne_session *sess, int isproxy, unsigned protomask,
     (*hdl)->creds = creds;
     (*hdl)->userdata = userdata;
     (*hdl)->next = NULL;
+    (*hdl)->attempt = 0;
 }
 
-static void auth_register_default(ne_session *sess, int isproxy,
-                                  const struct auth_class *ahc, const char *id,
-                                  ne_auth_creds creds, void *userdata) 
-{
-    unsigned protomask = NE_AUTH_BASIC|NE_AUTH_DIGEST;
-    
-    if (strcmp(ne_get_scheme(sess), "https") == 0 || isproxy) {
-        protomask |= NE_AUTH_NEGOTIATE;
-    }
-
-    auth_register(sess, isproxy, protomask, ahc, id, creds, userdata);
-}
-
-static const struct auth_protocol protocols[] = {
-    { NE_AUTH_BASIC, 10, "Basic",
-      basic_challenge, request_basic, NULL,
-      0 },
-    { NE_AUTH_DIGEST, 20, "Digest",
-      digest_challenge, request_digest, verify_digest_response,
-      0 },
-#ifdef HAVE_GSSAPI
-    { NE_AUTH_NEGOTIATE, 30, "Negotiate",
-      negotiate_challenge, request_negotiate, verify_negotiate_response,
-      AUTH_FLAG_OPAQUE_PARAM|AUTH_FLAG_VERIFY_NON40x },
-#endif
-#ifdef HAVE_SSPI
-    { NE_AUTH_NEGOTIATE, 30, "NTLM",
-      sspi_challenge, request_sspi, NULL,
-      AUTH_FLAG_OPAQUE_PARAM|AUTH_FLAG_VERIFY_NON40x },
-    { NE_AUTH_NEGOTIATE, 30, "Negotiate",
-      sspi_challenge, request_sspi, NULL,
-      AUTH_FLAG_OPAQUE_PARAM|AUTH_FLAG_VERIFY_NON40x },
-#endif
-    { 0 }
-};
-
 void ne_set_server_auth(ne_session *sess, ne_auth_creds creds, void *userdata)
 {
-    auth_register_default(sess, 0, &ah_server_class, HOOK_SERVER_ID,
-                          creds, userdata);
+    auth_register(sess, 0, NE_AUTH_DEFAULT, &ah_server_class, HOOK_SERVER_ID,
+                  creds, userdata);
 }
 
 void ne_set_proxy_auth(ne_session *sess, ne_auth_creds creds, void *userdata)
 {
-    auth_register_default(sess, 1, &ah_proxy_class, HOOK_PROXY_ID,
-                          creds, userdata);
+    auth_register(sess, 1, NE_AUTH_DEFAULT, &ah_proxy_class, HOOK_PROXY_ID,
+                  creds, userdata);
 }
 
 void ne_add_server_auth(ne_session *sess, unsigned protocol, 
