@@ -1,6 +1,6 @@
 /* 
    sitecopy, manage remote web sites.
-   Copyright (C) 1998-2002, Joe Orton <joe@manyfish.co.uk>.
+   Copyright (C) 1998-2004, Joe Orton <joe@manyfish.co.uk>.
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 
 */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #ifdef HAVE_STRING_H
@@ -32,6 +32,10 @@
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
 
+#ifdef HAVE_LANGINFO_H
+#include <langinfo.h>
+#endif
+
 #include <ctype.h>
 
 #include <ne_string.h>
@@ -56,18 +60,27 @@ static const struct {
     { "httpauth", NE_DBG_HTTPAUTH },
     { "httpbody", NE_DBG_HTTPBODY },
     { "cleartext", NE_DBG_HTTPPLAIN },
+    { "ssl", NE_DBG_SSL },
+    { "rsh", DEBUG_RSH },
+    { "sftp", DEBUG_SFTP },
     { NULL, 0 }
 };
 
 int map_debug_options(const char *opts, int *mask, char *errbuf)
 {
-    char **tokens = split_string(opts, ',', NULL, " ");
-    int tok, n, ret = 0, mapped;
+    char *token, *ptr, *orig;
+    int n, ret = 0, mapped;
+
+    ptr = orig = ne_strdup(opts);
+
+    do {
+        token = ne_token(&ptr, ',');
+        token = ne_shave(token, " ");
 
-    for (tok = 0; tokens[tok] != NULL; tok++) {
 	mapped = 0;
+        
 	for (n = 0; debug_map[n].name != NULL; n++) {
-	    if (strcasecmp(tokens[tok], debug_map[n].name) == 0) {
+	    if (strcasecmp(token, debug_map[n].name) == 0) {
 		ret |= debug_map[n].val;
 		mapped = 1;
 		break;
@@ -75,14 +88,14 @@ int map_debug_options(const char *opts, int *mask, char *errbuf)
 	}
 	if (!mapped) {
 	    memset(errbuf, 0, 20);
-	    strncpy(errbuf, tokens[tok], 19);
-	    split_string_free(tokens);
+	    strncpy(errbuf, token, 19);
+            ne_free(orig);
 	    return -1;
 	}
-    }
+    } while (ptr);
     
     *mask = ret;
-    split_string_free(tokens);
+    ne_free(orig);
     return 0;
 }
 #endif /* DEBUGGING */
@@ -100,4 +113,3 @@ char *strerror (errnum)
   return _("Unknown system error");
 }
 # endif /* HAVE_STRERROR */
-
