@@ -1,6 +1,6 @@
 /* 
    sitecopy WebDAV protocol driver module
-   Copyright (C) 2000-2006, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2000-2008, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -64,7 +64,11 @@ struct fetch_context {
 
 #define ENABLE_PROGRESS do { ne_set_progress(sess, site_sock_progress_cb, NULL); } while (0)
 
+#if NE_VERSION_MAJOR == 0 && NE_VERSION_MINOR > 26
+#define DISABLE_PROGRESS do { ne_set_notifier(sess, NULL, NULL); } while (0)
+#else
 #define DISABLE_PROGRESS do { ne_set_progress(sess, NULL, NULL); } while (0)
+#endif
 
 /* TODO:
  * not really sure whether we should be using an enum here... what
@@ -79,7 +83,7 @@ struct private {
 #define ELM_resourcetype (NE_PROPS_STATE_TOP + 1)
 #define ELM_collection (NE_PROPS_STATE_TOP + 2)
 
-/* The element definitinos for the complex prop handler. */
+/* The element definitions for the complex prop handler. */
 static const struct ne_xml_idmap fetch_elms[] = {
     { "DAV:", "resourcetype", ELM_resourcetype },
     { "DAV:", "collection", ELM_collection },
@@ -140,6 +144,7 @@ proxy_auth_cb(void *userdata, const char *realm, int attempt,
 		       username, password);
 }
 
+#if NE_VERSION_MINOR < 27
 static void notify_cb(void *userdata, ne_conn_status status, const char *info)
 {
 
@@ -155,6 +160,25 @@ static void notify_cb(void *userdata, ne_conn_status status, const char *info)
 
 #undef MAP
 }
+#else
+static void notify_status(void *userdata, ne_session_status status,
+                          const ne_session_status_info *info)
+{
+    switch (status) {
+    case ne_status_lookup:
+        fe_connection(fe_namelookup, info->lu.hostname);
+        break;
+    case ne_status_connecting:
+        fe_connection(fe_connecting, NULL);
+        break;
+    case ne_status_connected:
+        fe_connection(fe_connected, NULL);
+        break;
+    default:
+        break;
+    }
+}
+#endif
 
 static int h2s(ne_session *sess, int errcode)
 {
@@ -180,30 +204,34 @@ static int h2s(ne_session *sess, int errcode)
     }
 }
 
-/* Verify callback for when a trusted cert is not known. */
-static int verify_untrusted(void *userdata, int failures,
-                            const ne_ssl_certificate *cert)
+/* Callback invoked when SSL server cert verification fails. */
+static int verify_certificate(void *userdata, int failures,
+                              const ne_ssl_certificate *cert)
 {
     struct site *site = userdata;
 
-    if (fe_accept_cert(cert, failures))
+    /* If the server cert has not changed since the user accepted it,
+     * trust the cert, unless it has expired, in which case the user
+     * should get a warning. */
+    if (site->server_cert 
+        && ne_ssl_cert_cmp(cert, site->server_cert) == 0
+        && (failures & NE_SSL_EXPIRED) == 0) {
+        return 0;
+    }
+
+    if (fe_accept_cert(cert, failures)) {
+        /* Not accepted by user => fail verification. */
         return -1;
-    
-    /* TODO: how to handle a write error here? */
-    ne_ssl_cert_write(cert, site->certfile);
+    }
+
+    if (ne_ssl_cert_write(cert, site->certfile)) {
+        fe_warning(_("Could not write SSL certificate"),
+                   NULL, site->certfile);
+    }
 
     return 0;
 }
 
-/* Verify callback for when a trusted cert is known. */
-static int verify_trusted(void *userdata, int failures,
-                          const ne_ssl_certificate *cert)
-{
-    ne_ssl_certificate *expected = userdata;
-
-    return failures != NE_SSL_UNTRUSTED || ne_ssl_cert_cmp(expected, cert);
-}
-
 static int init(void **session, struct site *site)
 {
     ne_session *sess;
@@ -223,19 +251,21 @@ static int init(void **session, struct site *site)
 
     if (site->http_secure) {
         if (access(site->certfile, R_OK) == 0) {
-            ne_ssl_certificate *cert = ne_ssl_cert_read(site->certfile);
-            if (cert == NULL) {
+            site->server_cert = ne_ssl_cert_read(site->certfile);
+            if (site->server_cert == NULL) {
                 ne_set_error(sess, _("Could not load certificate `%s'."),
                              site->certfile);
                 return SITE_FAILED;
             }
-            ne_ssl_set_verify(sess, verify_trusted, cert);
-        } else {
-            ne_ssl_set_verify(sess, verify_untrusted, site);
         }
+        ne_ssl_set_verify(sess, verify_certificate, site);
     }
 
+#if NE_VERSION_MINOR < 27
     ne_set_status(sess, notify_cb, NULL);
+#else
+    ne_set_notifier(sess, notify_status, NULL);
+#endif
 
     if (site->http_limit) {
 #if NE_VERSION_MINOR > 25
