@@ -67,6 +67,7 @@ int ftp_open(ftp_session *sess); /* Performs the login procedure */
 
 int ftp_set_server(ftp_session *sess, struct site_host *server);
 void ftp_set_passive(ftp_session *sess, int use_passive);
+void ftp_set_usecwd(ftp_session *sess, int use_cwd);
 
 const char *ftp_get_error(ftp_session *sess);
 int ftp_finish(ftp_session *sess);
@@ -91,6 +92,6 @@ int ftp_fetch_modtimes(ftp_session *sess,
 int ftp_get_modtime(ftp_session *sess, const char *filename, time_t *modtime);
 
 int ftp_read_file(ftp_session *sess, const char *remotefile,
-		  sock_block_reader reader, void *userdata);
+		  ne_block_reader reader, void *userdata);
 
 #endif /* FTP_H */
