@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites.
-   Copyright (C) 1998-2006, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1998-2008, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -768,7 +768,7 @@ int site_readfiles(struct site *site)
 
 /* Initial size of directory stack, and amount it grows
  * each time we fill it. */
-#define DIRSTACKSIZE 128
+#define DIRSTACKSIZE (1024)
 
 void site_read_local_state(struct site *site)
 {
@@ -1359,7 +1359,7 @@ void site_flatlist(FILE *f, struct site *site)
     fprintf(f, "siteend|%s\n", site->remote_is_different?"changed":"unchanged");
 }
 
-void site_sock_progress_cb(void *userdata, off_t progress, off_t total)
+void site_sock_progress_cb(void *userdata, ne_off_t progress, ne_off_t total)
 {
     fe_transfer_progress(progress, total);
 }
