@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites. Stored state handling routines.
-   Copyright (C) 1999-2004, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2006, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -54,7 +54,8 @@
  */
 #define SITE_STATE_FILE_VERSION "1.0"
 
-#define EOL "\r\n"
+/* Used in stored.mode to indicate no mode known. */
+#define INVALID_MODE ((mode_t)-1)
 
 /* Opens the storage file for writing */
 FILE *site_open_storage_file(struct site *site) 
@@ -126,23 +127,23 @@ int site_write_stored_state(struct site *site)
 	return -1;
     }
 
-    fprintf(fp, "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" EOL);
-    fprintf(fp, "<sitestate version='" SITE_STATE_FILE_VERSION "'>" EOL);
-    fprintf(fp, "<options>" EOL);
+    fprintf(fp, "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
+    fprintf(fp, "<sitestate version='" SITE_STATE_FILE_VERSION "'>\n");
+    fprintf(fp, "<options>\n");
     fprintf(fp, " <saved-by package='" PACKAGE_NAME "'"
-	    " version='" PACKAGE_VERSION "'/>" EOL);
+	    " version='" PACKAGE_VERSION "'/>\n");
     if (site->state_method == state_checksum) {
 	/* For forwards-compatibility */
-	fprintf(fp, " <checksum-algorithm><checksum-MD5/></checksum-algorithm>" EOL);
+	fprintf(fp, " <checksum-algorithm><checksum-MD5/></checksum-algorithm>\n");
     }
-    fprintf(fp, " <state-method><state-%s/></state-method>" EOL,
+    fprintf(fp, " <state-method><state-%s/></state-method>\n",
 	     (site->state_method==state_checksum)?"checksum":"timesize");
     if (site->safemode) {
-	fprintf(fp, " <safemode/>" EOL);
+	fprintf(fp, " <safemode/>\n");
     }
-    fprintf(fp, " <escaped-filenames/>" EOL);
-    fprintf(fp, "</options>" EOL);
-    fprintf(fp, "<items>" EOL);
+    fprintf(fp, " <escaped-filenames/>\n");
+    fprintf(fp, "</options>\n");
+    fprintf(fp, "<items>\n");
     /* Now write out the items */
     for (current = site->files; current!=NULL; current = current->next) {
 	char *fname;
@@ -155,8 +156,10 @@ int site_write_stored_state(struct site *site)
         fname = fn_escape(current->stored.filename);
 	fprintf(fp, "<filename>%s</filename>\n", fname);
         ne_free(fname);
-        fprintf(fp, "<protection>%03o</protection>", 
-                current->stored.mode); /* three-digit octal */
+        if (current->stored.mode != INVALID_MODE) {
+            fprintf(fp, "<protection>%03o</protection>", 
+                    current->stored.mode); /* three-digit octal */
+        }
 	switch (current->type) {
 	case file_link:
 	    fprintf(fp, "<linktarget>%s</linktarget>", 
@@ -186,16 +189,14 @@ int site_write_stored_state(struct site *site)
 	    /* nothing to do */
 	    break;
 	}
-	fprintf(fp, "</item>" EOL);
+	fprintf(fp, "</item>\n");
     }
-    fprintf(fp, "</items>" EOL);
-    fprintf(fp, "</sitestate>" EOL);
+    fprintf(fp, "</items>\n");
+    fprintf(fp, "</sitestate>\n");
     site->stored_state_method = site->state_method;
     return site_close_storage_file(site);
 }
 
-#undef EOL
-
 /* neon ne_xml-based XML parsing */
 
 #define ELM_BASE 500
@@ -276,7 +277,11 @@ static int start_element(void *userdata, int parent,
         /* Clear current stored state */
         memset(&doc->stored, 0, sizeof doc->stored);
         /* Initialize perms bits to invalid state */
-        doc->stored.mode = -1;
+        doc->stored.mode = INVALID_MODE;
+    }
+
+    if (state == SITE_ELM_ascii) {
+        doc->truth = 0;
     }
 
     return state;
@@ -337,10 +342,6 @@ static int end_element(void *userdata, int state,
     case SITE_ELM_size:
 	doc->stored.size = strtol(cdata, NULL, 10);
 	if (doc->stored.size == LONG_MAX) {
-            ne_snprintf(err, sizeof err, _("Size overflow (%s) at line %d"),
-                        cdata, ne_xml_currentline(doc->parser));
-            ne_xml_set_error(doc->parser, err);
-	    return -1;
         }
 	break;
     case SITE_ELM_protection:
@@ -349,13 +350,13 @@ static int end_element(void *userdata, int state,
     case SITE_ELM_server_modtime:
 	doc->server.time = strtol(cdata, NULL, 10);
 	if (doc->server.time == LONG_MIN || doc->server.time == LONG_MAX)
-	    return -1;
+            goto overflow_err;
 	doc->server.exists = true;
 	break;
     case SITE_ELM_modtime:
 	doc->stored.time = strtol(cdata, NULL, 10);
 	if (doc->stored.time == LONG_MIN || doc->stored.time == LONG_MAX)
-	    return -1;
+            goto overflow_err;
 	break;
     case SITE_ELM_true:
 	doc->truth = 1;
@@ -365,8 +366,12 @@ static int end_element(void *userdata, int state,
 	break;
     case SITE_ELM_ascii:
 	if (doc->truth) {
-	    doc->stored.ascii = (doc->truth==1);
+	    doc->stored.ascii = doc->truth == 1;
 	} else {
+            ne_snprintf(err, sizeof err, _("Boolean missing in 'ascii' "
+                                           "at line %d"),
+                        ne_xml_currentline(doc->parser));
+            ne_xml_set_error(doc->parser, err);
 	    return -1;
 	}
 	break;
@@ -387,7 +392,11 @@ static int end_element(void *userdata, int state,
     }
 
     return 0;
-
+overflow_err:
+    ne_snprintf(err, sizeof err, _("Size overflow (%s) in '%s' at line %d"),
+                cdata, name, ne_xml_currentline(doc->parser));
+    ne_xml_set_error(doc->parser, err);
+    return -1;
 }
 
 /* Read a new XML-format state storage file */
